@@ -1,3 +1,4 @@
+/* -*- Mode: c; tab-width: 8; c-basic-offset: 4; indent-tabs-mode: t; -*- */
 /*
  * Copyright © 2006 Keith Packard
  * Copyright © 2007 Adrian Johnson
@@ -59,6 +60,10 @@ typedef struct {
     cairo_region_t fallback_region;
     cairo_box_t page_bbox;
 
+    cairo_bool_t create_region_ids;
+    unsigned source_region_id;
+    unsigned mask_region_id;
+
     cairo_bool_t has_ctm;
     cairo_matrix_t ctm;
 
@@ -118,7 +123,7 @@ attach_proxy (cairo_surface_t *source,
 {
     struct proxy *proxy;
 
-    proxy = malloc (sizeof (*proxy));
+    proxy = _cairo_malloc (sizeof (*proxy));
     if (unlikely (proxy == NULL))
 	return _cairo_surface_create_in_error (CAIRO_STATUS_NO_MEMORY);
 
@@ -257,15 +262,19 @@ _add_operation (cairo_analysis_surface_t *surface,
 static cairo_int_status_t
 _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
 				    const cairo_pattern_t    *pattern,
-				    cairo_rectangle_int_t    *extents)
+				    cairo_rectangle_int_t    *extents,
+				    unsigned int             *regions_id,
+				    cairo_analysis_source_t   source_type)
 {
     const cairo_surface_pattern_t *surface_pattern;
     cairo_analysis_surface_t *tmp;
     cairo_surface_t *source, *proxy;
     cairo_matrix_t p2d;
-    cairo_int_status_t status, analysis_status;
+    cairo_int_status_t status;
+    cairo_int_status_t analysis_status = CAIRO_INT_STATUS_SUCCESS;
     cairo_bool_t surface_is_unbounded;
     cairo_bool_t unused;
+    cairo_bool_t replay_all;
 
     assert (pattern->type == CAIRO_PATTERN_TYPE_SURFACE);
     surface_pattern = (const cairo_surface_pattern_t *) pattern;
@@ -279,9 +288,9 @@ _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
     }
 
     tmp = (cairo_analysis_surface_t *)
-	_cairo_analysis_surface_create (surface->target);
+	_cairo_analysis_surface_create (surface->target, surface->create_region_ids);
     if (unlikely (tmp->base.status)) {
-	status =tmp->base.status;
+	status = tmp->base.status;
 	goto cleanup1;
     }
     proxy = attach_proxy (source, &tmp->base);
@@ -291,16 +300,63 @@ _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
     assert (status == CAIRO_INT_STATUS_SUCCESS);
     _cairo_analysis_surface_set_ctm (&tmp->base, &p2d);
 
-
     source = _cairo_surface_get_source (source, NULL);
     surface_is_unbounded = (pattern->extend == CAIRO_EXTEND_REPEAT
-				     || pattern->extend == CAIRO_EXTEND_REFLECT);
-    status = _cairo_recording_surface_replay_and_create_regions (source,
+			    || pattern->extend == CAIRO_EXTEND_REFLECT);
+
+    if (surface->create_region_ids) {
+	status = _cairo_recording_surface_region_array_attach (source, regions_id);
+	if (unlikely (status))
+	    goto cleanup2;
+    }
+
+    replay_all = FALSE;
+    if (surface->target->backend->analyze_recording_surface) {
+	status = surface->target->backend->analyze_recording_surface (
+	    surface->target,
+	    surface_pattern,
+	    surface->create_region_ids ? *regions_id : 0,
+	    source_type,
+	    TRUE);
+        if (status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
+	    /* Ensure all commands are replayed even if previously
+	     * replayed and assigned to a region.*/
+            replay_all = TRUE;
+            status = CAIRO_INT_STATUS_SUCCESS;
+        }
+        if (unlikely (status))
+            goto cleanup3;
+    }
+
+    if (surface->create_region_ids) {
+	status = _cairo_recording_surface_replay_and_create_regions (source,
+								     *regions_id,
+								     &pattern->matrix,
+								     &tmp->base,
+								     surface_is_unbounded,
+								     replay_all);
+	if (unlikely (status))
+	    goto cleanup3;
+    } else {
+	status = _cairo_recording_surface_replay_with_transform (source,
 								 &pattern->matrix,
 								 &tmp->base,
-								 surface_is_unbounded);
-    if (unlikely (status))
-	goto cleanup2;
+								 surface_is_unbounded,
+								 replay_all);
+	if (unlikely (status))
+	    goto cleanup3;
+    }
+
+    if (surface->target->backend->analyze_recording_surface) {
+	status = surface->target->backend->analyze_recording_surface (
+	    surface->target,
+	    surface_pattern,
+	    surface->create_region_ids ? *regions_id : 0,
+	    source_type,
+	    FALSE);
+        if (unlikely (status))
+            goto cleanup3;
+    }
 
     /* black background or mime data fills entire extents */
     if (!(source->content & CAIRO_CONTENT_ALPHA) || _cairo_surface_has_mime_image (source)) {
@@ -316,7 +372,7 @@ _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
 	    if (status == CAIRO_INT_STATUS_IMAGE_FALLBACK)
 		status = CAIRO_INT_STATUS_SUCCESS;
 	    if (unlikely (status))
-		goto cleanup2;
+		goto cleanup3;
 	}
     }
 
@@ -340,6 +396,10 @@ _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
 	_cairo_box_round_to_rectangle (&tmp->page_bbox, extents);
     }
 
+  cleanup3:
+    if (surface->create_region_ids && unlikely (status)) {
+	_cairo_recording_surface_region_array_remove (source, *regions_id);
+    }
   cleanup2:
     detach_proxy (proxy);
   cleanup1:
@@ -347,8 +407,8 @@ _analyze_recording_surface_pattern (cairo_analysis_surface_t *surface,
 
     if (unlikely (status))
 	return status;
-
-    return analysis_status;
+    else
+	return analysis_status;
 }
 
 static cairo_status_t
@@ -411,6 +471,8 @@ _cairo_analysis_surface_paint (void			*abstract_surface,
     cairo_int_status_t	     backend_status;
     cairo_rectangle_int_t  extents;
 
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
     if (surface->target->backend->paint == NULL) {
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
     } else {
@@ -426,7 +488,11 @@ _cairo_analysis_surface_paint (void			*abstract_surface,
 					       &extents);
     if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
 	cairo_rectangle_int_t rec_extents;
-	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	backend_status = _analyze_recording_surface_pattern (surface,
+							     source,
+							     &rec_extents,
+							     &surface->source_region_id,
+							     CAIRO_ANALYSIS_SOURCE_PAINT);
 	_cairo_rectangle_intersect (&extents, &rec_extents);
     }
 
@@ -444,6 +510,8 @@ _cairo_analysis_surface_mask (void			*abstract_surface,
     cairo_int_status_t	      backend_status;
     cairo_rectangle_int_t   extents;
 
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
     if (surface->target->backend->mask == NULL) {
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
     } else {
@@ -467,7 +535,11 @@ _cairo_analysis_surface_mask (void			*abstract_surface,
 	    src_surface = _cairo_surface_get_source (src_surface, NULL);
 	    if (_cairo_surface_is_recording (src_surface)) {
 		backend_source_status =
-		    _analyze_recording_surface_pattern (surface, source, &rec_extents);
+		    _analyze_recording_surface_pattern (surface,
+							source,
+							&rec_extents,
+							&surface->source_region_id,
+							CAIRO_ANALYSIS_SOURCE_MASK);
 		if (_cairo_int_status_is_error (backend_source_status))
 		    return backend_source_status;
 
@@ -480,7 +552,11 @@ _cairo_analysis_surface_mask (void			*abstract_surface,
 	    mask_surface = _cairo_surface_get_source (mask_surface, NULL);
 	    if (_cairo_surface_is_recording (mask_surface)) {
 		backend_mask_status =
-		    _analyze_recording_surface_pattern (surface, mask, &rec_extents);
+		    _analyze_recording_surface_pattern (surface,
+							mask,
+							&rec_extents,
+							&surface->mask_region_id,
+							CAIRO_ANALYSIS_MASK_MASK);
 		if (_cairo_int_status_is_error (backend_mask_status))
 		    return backend_mask_status;
 
@@ -519,6 +595,8 @@ _cairo_analysis_surface_stroke (void			   *abstract_surface,
     cairo_int_status_t	     backend_status;
     cairo_rectangle_int_t    extents;
 
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
     if (surface->target->backend->stroke == NULL) {
 	backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
     } else {
@@ -537,7 +615,11 @@ _cairo_analysis_surface_stroke (void			   *abstract_surface,
 					       &extents);
     if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
 	cairo_rectangle_int_t rec_extents;
-	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	backend_status = _analyze_recording_surface_pattern (surface,
+							     source,
+							     &rec_extents,
+							     &surface->source_region_id,
+							     CAIRO_ANALYSIS_SOURCE_STROKE);
 	_cairo_rectangle_intersect (&extents, &rec_extents);
     }
 
@@ -589,7 +671,11 @@ _cairo_analysis_surface_fill (void			*abstract_surface,
 					       &extents);
     if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
 	cairo_rectangle_int_t rec_extents;
-	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	backend_status = _analyze_recording_surface_pattern (surface,
+							     source,
+							     &rec_extents,
+							     &surface->source_region_id,
+							     CAIRO_ANALYSIS_SOURCE_FILL);
 	_cairo_rectangle_intersect (&extents, &rec_extents);
     }
 
@@ -618,6 +704,9 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
     cairo_int_status_t	     status, backend_status;
     cairo_rectangle_int_t    extents, glyph_extents;
 
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
+
     /* Adapted from _cairo_surface_show_glyphs */
     if (surface->target->backend->show_glyphs != NULL) {
 	backend_status =
@@ -653,7 +742,11 @@ _cairo_analysis_surface_show_glyphs (void		  *abstract_surface,
 					       &extents);
     if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
 	cairo_rectangle_int_t rec_extents;
-	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	backend_status = _analyze_recording_surface_pattern (surface,
+							     source,
+							     &rec_extents,
+							     &surface->source_region_id,
+							     CAIRO_ANALYSIS_SOURCE_SHOW_GLYPHS);
 	_cairo_rectangle_intersect (&extents, &rec_extents);
     }
 
@@ -698,6 +791,9 @@ _cairo_analysis_surface_show_text_glyphs (void			    *abstract_surface,
     cairo_int_status_t	     status, backend_status;
     cairo_rectangle_int_t    extents, glyph_extents;
 
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
+
     /* Adapted from _cairo_surface_show_glyphs */
     backend_status = CAIRO_INT_STATUS_UNSUPPORTED;
     if (surface->target->backend->show_text_glyphs != NULL) {
@@ -731,7 +827,11 @@ _cairo_analysis_surface_show_text_glyphs (void			    *abstract_surface,
 					       &extents);
     if (backend_status == CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN) {
 	cairo_rectangle_int_t rec_extents;
-	backend_status = _analyze_recording_surface_pattern (surface, source, &rec_extents);
+	_analyze_recording_surface_pattern (surface,
+					    source,
+					    &rec_extents,
+					    &surface->source_region_id,
+					    CAIRO_ANALYSIS_SOURCE_SHOW_GLYPHS);
 	_cairo_rectangle_intersect (&extents, &rec_extents);
     }
 
@@ -754,30 +854,49 @@ static cairo_int_status_t
 _cairo_analysis_surface_tag (void	                *abstract_surface,
 			     cairo_bool_t                begin,
 			     const char                 *tag_name,
-			     const char                 *attributes,
-			     const cairo_pattern_t	*source,
-			     const cairo_stroke_style_t	*stroke_style,
-			     const cairo_matrix_t	*ctm,
-			     const cairo_matrix_t	*ctm_inverse,
-			     const cairo_clip_t	        *clip)
+			     const char                 *attributes)
 {
     cairo_analysis_surface_t *surface = abstract_surface;
     cairo_int_status_t	     backend_status;
 
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
     backend_status = CAIRO_INT_STATUS_SUCCESS;
     if (surface->target->backend->tag != NULL) {
 	backend_status =
 	    surface->target->backend->tag (surface->target,
 					   begin,
 					   tag_name,
-					   attributes,
-					   source,
-					   stroke_style,
-					   ctm,
-					   ctm_inverse,
-					   clip);
-	if (_cairo_int_status_is_error (backend_status))
-	    return backend_status;
+					   attributes);
+        if (backend_status == CAIRO_INT_STATUS_SUCCESS)
+            surface->has_supported = TRUE;
+    }
+
+    return backend_status;
+}
+
+static cairo_bool_t
+_cairo_analysis_surface_supports_color_glyph (void                 *abstract_surface,
+                                              cairo_scaled_font_t  *scaled_font,
+                                              unsigned long         glyph_index)
+{
+    return TRUE;
+}
+
+static cairo_int_status_t
+_cairo_analysis_surface_command_id (void                 *abstract_surface,
+				    unsigned int          recording_id,
+				    unsigned int          command_id)
+{
+    cairo_analysis_surface_t *surface = abstract_surface;
+    cairo_int_status_t backend_status;
+
+    backend_status = CAIRO_INT_STATUS_SUCCESS;
+    if (surface->target->backend->command_id != NULL) {
+	backend_status =
+	    surface->target->backend->command_id (surface->target,
+						  recording_id,
+						  command_id);
     }
 
     return backend_status;
@@ -817,11 +936,15 @@ static const cairo_surface_backend_t cairo_analysis_surface_backend = {
     _cairo_analysis_surface_has_show_text_glyphs,
     _cairo_analysis_surface_show_text_glyphs,
     NULL, /* get_supported_mime_types */
-    _cairo_analysis_surface_tag
+    _cairo_analysis_surface_tag,
+    _cairo_analysis_surface_supports_color_glyph,
+    NULL, /* analyze_recording_surface */
+    _cairo_analysis_surface_command_id,
 };
 
 cairo_surface_t *
-_cairo_analysis_surface_create (cairo_surface_t		*target)
+_cairo_analysis_surface_create (cairo_surface_t		*target,
+				cairo_bool_t             create_region_ids)
 {
     cairo_analysis_surface_t *surface;
     cairo_status_t status;
@@ -830,7 +953,7 @@ _cairo_analysis_surface_create (cairo_surface_t		*target)
     if (unlikely (status))
 	return _cairo_surface_create_in_error (status);
 
-    surface = malloc (sizeof (cairo_analysis_surface_t));
+    surface = _cairo_malloc (sizeof (cairo_analysis_surface_t));
     if (unlikely (surface == NULL))
 	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_NO_MEMORY));
 
@@ -850,6 +973,10 @@ _cairo_analysis_surface_create (cairo_surface_t		*target)
     surface->has_supported = FALSE;
     surface->has_unsupported = FALSE;
 
+    surface->create_region_ids = create_region_ids;
+    surface->source_region_id = 0;
+    surface->mask_region_id = 0;
+
     _cairo_region_init (&surface->supported_region);
     _cairo_region_init (&surface->fallback_region);
 
@@ -927,58 +1054,119 @@ _cairo_analysis_surface_get_bounding_box (cairo_surface_t *abstract_surface,
     *bbox = surface->page_bbox;
 }
 
+unsigned int
+_cairo_analysis_surface_get_source_region_id (cairo_surface_t *abstract_surface)
+{
+    cairo_analysis_surface_t	*surface = (cairo_analysis_surface_t *) abstract_surface;
+
+    return surface->source_region_id;
+}
+
+unsigned int
+_cairo_analysis_surface_get_mask_region_id (cairo_surface_t *abstract_surface)
+{
+    cairo_analysis_surface_t	*surface = (cairo_analysis_surface_t *) abstract_surface;
+
+    return surface->mask_region_id;
+}
+
+
 /* null surface type: a surface that does nothing (has no side effects, yay!) */
 
 static cairo_int_status_t
-_return_success (void)
+_paint_return_success (void			*surface,
+		       cairo_operator_t		 op,
+		       const cairo_pattern_t	*source,
+		       const cairo_clip_t	*clip)
 {
-    return CAIRO_STATUS_SUCCESS;
+    if (source->type == CAIRO_PATTERN_TYPE_SURFACE) {
+        cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) source;
+        if (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING)
+            return CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN;
+    }
+
+    return CAIRO_INT_STATUS_SUCCESS;
 }
 
-/* These typedefs are just to silence the compiler... */
-typedef cairo_int_status_t
-(*_paint_func)			(void			*surface,
-			         cairo_operator_t	 op,
-				 const cairo_pattern_t	*source,
-				 const cairo_clip_t		*clip);
+static cairo_int_status_t
+_mask_return_success (void			*surface,
+		      cairo_operator_t		 op,
+		      const cairo_pattern_t	*source,
+		      const cairo_pattern_t	*mask,
+		      const cairo_clip_t	*clip)
+{
+    if (source->type == CAIRO_PATTERN_TYPE_SURFACE) {
+        cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) source;
+        if (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING)
+            return CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN;
+    }
 
-typedef cairo_int_status_t
-(*_mask_func)			(void			*surface,
-			         cairo_operator_t	 op,
-				 const cairo_pattern_t	*source,
-				 const cairo_pattern_t	*mask,
-				 const cairo_clip_t		*clip);
+    if (mask->type == CAIRO_PATTERN_TYPE_SURFACE) {
+        cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) mask;
+        if (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING)
+            return CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN;
+    }
 
-typedef cairo_int_status_t
-(*_stroke_func)			(void			*surface,
-			         cairo_operator_t	 op,
-				 const cairo_pattern_t	*source,
-				 const cairo_path_fixed_t	*path,
-				 const cairo_stroke_style_t	*style,
-				 const cairo_matrix_t		*ctm,
-				 const cairo_matrix_t		*ctm_inverse,
-				 double			 tolerance,
-				 cairo_antialias_t	 antialias,
-				 const cairo_clip_t		*clip);
+    return CAIRO_INT_STATUS_SUCCESS;
+}
 
-typedef cairo_int_status_t
-(*_fill_func)			(void			*surface,
-			         cairo_operator_t	 op,
-				 const cairo_pattern_t	*source,
-				 const cairo_path_fixed_t	*path,
-				 cairo_fill_rule_t	 fill_rule,
-				 double			 tolerance,
-				 cairo_antialias_t	 antialias,
-				 const cairo_clip_t		*clip);
+static cairo_int_status_t
+_stroke_return_success (void				*surface,
+			cairo_operator_t		 op,
+			const cairo_pattern_t		*source,
+			const cairo_path_fixed_t	*path,
+			const cairo_stroke_style_t	*style,
+			const cairo_matrix_t		*ctm,
+			const cairo_matrix_t		*ctm_inverse,
+			double				 tolerance,
+			cairo_antialias_t		 antialias,
+			const cairo_clip_t		*clip)
+{
+    if (source->type == CAIRO_PATTERN_TYPE_SURFACE) {
+        cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) source;
+        if (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING)
+            return CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN;
+    }
 
-typedef cairo_int_status_t
-(*_show_glyphs_func)		(void			*surface,
-			         cairo_operator_t	 op,
-				 const cairo_pattern_t	*source,
-				 cairo_glyph_t		*glyphs,
-				 int			 num_glyphs,
-				 cairo_scaled_font_t	*scaled_font,
-				 const cairo_clip_t		*clip);
+    return CAIRO_INT_STATUS_SUCCESS;
+}
+
+static cairo_int_status_t
+_fill_return_success (void			*surface,
+		      cairo_operator_t		 op,
+		      const cairo_pattern_t	*source,
+		      const cairo_path_fixed_t	*path,
+		      cairo_fill_rule_t		 fill_rule,
+		      double			 tolerance,
+		      cairo_antialias_t		 antialias,
+		      const cairo_clip_t	*clip)
+{
+    if (source->type == CAIRO_PATTERN_TYPE_SURFACE) {
+        cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) source;
+        if (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING)
+            return CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN;
+    }
+
+    return CAIRO_INT_STATUS_SUCCESS;
+}
+
+static cairo_int_status_t
+_show_glyphs_return_success (void			*surface,
+			     cairo_operator_t		 op,
+			     const cairo_pattern_t	*source,
+			     cairo_glyph_t		*glyphs,
+			     int			 num_glyphs,
+			     cairo_scaled_font_t	*scaled_font,
+			     const cairo_clip_t		*clip)
+{
+    if (source->type == CAIRO_PATTERN_TYPE_SURFACE) {
+        cairo_surface_pattern_t *surface_pattern = (cairo_surface_pattern_t *) source;
+        if (surface_pattern->surface->type == CAIRO_SURFACE_TYPE_RECORDING)
+            return CAIRO_INT_STATUS_ANALYZE_RECORDING_SURFACE_PATTERN;
+    }
+
+    return CAIRO_INT_STATUS_SUCCESS;
+}
 
 static const cairo_surface_backend_t cairo_null_surface_backend = {
     CAIRO_INTERNAL_SURFACE_TYPE_NULL,
@@ -1005,14 +1193,19 @@ static const cairo_surface_backend_t cairo_null_surface_backend = {
     NULL, /* flush */
     NULL, /* mark_dirty_rectangle */
 
-    (_paint_func) _return_success,	    /* paint */
-    (_mask_func) _return_success,	    /* mask */
-    (_stroke_func) _return_success,	    /* stroke */
-    (_fill_func) _return_success,	    /* fill */
+    _paint_return_success,	    /* paint */
+    _mask_return_success,	    /* mask */
+    _stroke_return_success,	    /* stroke */
+    _fill_return_success,	    /* fill */
     NULL, /* fill_stroke */
-    (_show_glyphs_func) _return_success,    /* show_glyphs */
+    _show_glyphs_return_success,    /* show_glyphs */
     NULL, /* has_show_text_glyphs */
-    NULL  /* show_text_glyphs */
+    NULL, /* show_text_glyphs */
+    NULL, /* get_supported_mime_types */
+    NULL, /* tag */
+    NULL, /* supports_color_glyph */
+    NULL, /* analyze_recording_surface */
+    NULL, /* command_id*/
 };
 
 cairo_surface_t *
@@ -1020,7 +1213,7 @@ _cairo_null_surface_create (cairo_content_t content)
 {
     cairo_surface_t *surface;
 
-    surface = malloc (sizeof (cairo_surface_t));
+    surface = _cairo_malloc (sizeof (cairo_surface_t));
     if (unlikely (surface == NULL)) {
 	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_NO_MEMORY));
     }
