@@ -107,12 +107,13 @@ struct _cairo_scaled_font {
     cairo_font_extents_t fs_extents; /* font space */
 
     /* The mutex protects modification to all subsequent fields. */
-    cairo_mutex_t mutex;
+    cairo_recursive_mutex_t mutex;
 
     cairo_hash_table_t *glyphs;
     cairo_list_t glyph_pages;
     cairo_bool_t cache_frozen;
     cairo_bool_t global_cache_frozen;
+    cairo_array_t recording_surfaces_to_free; /* array of cairo_surface_t* */
 
     cairo_list_t dev_privates;
 
@@ -148,14 +149,17 @@ struct _cairo_scaled_glyph {
     cairo_list_t            dev_privates;
 
     cairo_color_t           foreground_color;   /* only used for color glyphs */
-    /* TRUE if the color_surface required the foreground_color to render. */
-    unsigned                uses_foreground_color : 1;
 
-    /* TRUE if this is not a color glyph, FALSE if is a color glyph or unknown.  */
-    unsigned                not_color_glyph : 1;
+    /* TRUE if the recording_surface used the foreground_source to render. */
+    unsigned                recording_uses_foreground_color : 1;
 
-    /* TRUE if recording_surface is a color glyph */
-    unsigned                recording_is_color : 1;
+    /* TRUE if the recording surface uses the foreground marker. */
+    unsigned                recording_uses_foreground_marker : 1;
+
+    /* TRUE if color_glyph specifies if glyph is color or non color, FALSE if glyph color type unknown. */
+    unsigned                color_glyph_set : 1;
+
+    unsigned                color_glyph : 1;
 };
 
 struct _cairo_scaled_glyph_private {
