@@ -50,12 +50,6 @@
 #include "cairo-surface-backend-private.h"
 #include "cairo-compositor-private.h"
 
-#if CAIRO_HAS_XLIB_XCB_FUNCTIONS
-slim_hidden_proto (cairo_xcb_surface_create);
-slim_hidden_proto (cairo_xcb_surface_create_for_bitmap);
-slim_hidden_proto (cairo_xcb_surface_create_with_xrender_format);
-#endif
-
 /**
  * SECTION:cairo-xcb
  * @Title: XCB Surfaces
@@ -155,7 +149,7 @@ _cairo_xcb_surface_create_similar (void			*abstract_other,
     }
 
     if (unlikely (surface->base.status))
-	_cairo_xcb_connection_free_pixmap (connection, pixmap);
+	xcb_free_pixmap (connection->xcb_connection, pixmap);
 
     _cairo_xcb_connection_release (connection);
 
@@ -220,7 +214,7 @@ _cairo_xcb_surface_finish (void *abstract_surface)
 	}
 
 	if (surface->owns_pixmap)
-	    _cairo_xcb_connection_free_pixmap (surface->connection, surface->drawable);
+	    xcb_free_pixmap (surface->connection->xcb_connection, surface->drawable);
 	_cairo_xcb_connection_release (surface->connection);
     }
 
@@ -423,7 +417,7 @@ _get_image (cairo_xcb_surface_t		 *surface,
 						 pixmap,
 						 0, 0,
 						 width, height);
-	_cairo_xcb_connection_free_pixmap (connection, pixmap);
+	xcb_free_pixmap (connection->xcb_connection, pixmap);
     }
 
     if (unlikely (reply == NULL)) {
@@ -1075,7 +1069,7 @@ _cairo_xcb_surface_create_internal (cairo_xcb_screen_t		*screen,
 {
     cairo_xcb_surface_t *surface;
 
-    surface = malloc (sizeof (cairo_xcb_surface_t));
+    surface = _cairo_malloc (sizeof (cairo_xcb_surface_t));
     if (unlikely (surface == NULL))
 	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_NO_MEMORY));
 
@@ -1236,9 +1230,6 @@ cairo_xcb_surface_create (xcb_connection_t  *connection,
 					       xrender_format,
 					       width, height);
 }
-#if CAIRO_HAS_XLIB_XCB_FUNCTIONS
-slim_hidden_def (cairo_xcb_surface_create);
-#endif
 
 /**
  * cairo_xcb_surface_create_for_bitmap:
@@ -1287,9 +1278,6 @@ cairo_xcb_surface_create_for_bitmap (xcb_connection_t	*connection,
 					       cairo_xcb_screen->connection->standard_formats[CAIRO_FORMAT_A1],
 					       width, height);
 }
-#if CAIRO_HAS_XLIB_XCB_FUNCTIONS
-slim_hidden_def (cairo_xcb_surface_create_for_bitmap);
-#endif
 
 /**
  * cairo_xcb_surface_create_with_xrender_format:
@@ -1379,9 +1367,6 @@ cairo_xcb_surface_create_with_xrender_format (xcb_connection_t	    *connection,
 					       format->id,
 					       width, height);
 }
-#if CAIRO_HAS_XLIB_XCB_FUNCTIONS
-slim_hidden_def (cairo_xcb_surface_create_with_xrender_format);
-#endif
 
 /* This does the necessary fixup when a surface's drawable or size changed. */
 static void
@@ -1451,9 +1436,6 @@ cairo_xcb_surface_set_size (cairo_surface_t *abstract_surface,
     surface->width  = width;
     surface->height = height;
 }
-#if CAIRO_HAS_XLIB_XCB_FUNCTIONS
-slim_hidden_def (cairo_xcb_surface_set_size);
-#endif
 
 /**
  * cairo_xcb_surface_set_drawable:
@@ -1526,6 +1508,3 @@ cairo_xcb_surface_set_drawable (cairo_surface_t *abstract_surface,
     surface->width  = width;
     surface->height = height;
 }
-#if CAIRO_HAS_XLIB_XCB_FUNCTIONS
-slim_hidden_def (cairo_xcb_surface_set_drawable);
-#endif
