@@ -37,15 +37,6 @@
  *	Vladimir Vukicevic <vladimir@pobox.com>
  */
 
-#define WIN32_LEAN_AND_MEAN
-/* We require Windows 2000 features such as ETO_PDY */
-#if !defined(WINVER) || (WINVER < 0x0500)
-# define WINVER 0x0500
-#endif
-#if !defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x0500)
-# define _WIN32_WINNT 0x0500
-#endif
-
 #include "cairoint.h"
 
 #include "cairo-clip-private.h"
@@ -129,6 +120,8 @@ _create_dc_and_bitmap (cairo_win32_display_surface_t *surface,
     case CAIRO_FORMAT_INVALID:
     case CAIRO_FORMAT_RGB16_565:
     case CAIRO_FORMAT_RGB30:
+    case CAIRO_FORMAT_RGB96F:
+    case CAIRO_FORMAT_RGBA128F:
 	return _cairo_error (CAIRO_STATUS_INVALID_FORMAT);
     case CAIRO_FORMAT_ARGB32:
     case CAIRO_FORMAT_RGB24:
@@ -164,6 +157,8 @@ _create_dc_and_bitmap (cairo_win32_display_surface_t *surface,
     case CAIRO_FORMAT_INVALID:
     case CAIRO_FORMAT_RGB16_565:
     case CAIRO_FORMAT_RGB30:
+    case CAIRO_FORMAT_RGB96F:
+    case CAIRO_FORMAT_RGBA128F:
 	ASSERT_NOT_REACHED;
     /* We can't create real RGB24 bitmaps because something seems to
      * break if we do, especially if we don't set up an image
@@ -214,10 +209,10 @@ _create_dc_and_bitmap (cairo_win32_display_surface_t *surface,
 	goto FAIL;
 
     surface->bitmap = CreateDIBSection (surface->win32.dc,
-			                bitmap_info,
-			                DIB_RGB_COLORS,
-			                &bits,
-			                NULL, 0);
+					bitmap_info,
+					DIB_RGB_COLORS,
+					&bits,
+					NULL, 0);
     if (!surface->bitmap)
 	goto FAIL;
 
@@ -242,6 +237,8 @@ _create_dc_and_bitmap (cairo_win32_display_surface_t *surface,
 	case CAIRO_FORMAT_INVALID:
 	case CAIRO_FORMAT_RGB16_565:
 	case CAIRO_FORMAT_RGB30:
+	case CAIRO_FORMAT_RGB96F:
+	case CAIRO_FORMAT_RGBA128F:
 	    ASSERT_NOT_REACHED;
 	case CAIRO_FORMAT_ARGB32:
 	case CAIRO_FORMAT_RGB24:
@@ -258,7 +255,7 @@ _create_dc_and_bitmap (cairo_win32_display_surface_t *surface,
 	}
     }
 
-    surface->win32.flags = _cairo_win32_flags_for_dc (surface->win32.dc);
+    surface->win32.flags = _cairo_win32_flags_for_dc (surface->win32.dc, format);
 
     return CAIRO_STATUS_SUCCESS;
 
@@ -289,8 +286,8 @@ _create_dc_and_bitmap (cairo_win32_display_surface_t *surface,
 static cairo_surface_t *
 _cairo_win32_display_surface_create_for_dc (HDC             original_dc,
 					    cairo_format_t  format,
-					    int	            width,
-					    int	            height)
+					    int		    width,
+					    int		    height)
 {
     cairo_status_t status;
     cairo_device_t *device;
@@ -298,7 +295,7 @@ _cairo_win32_display_surface_create_for_dc (HDC             original_dc,
     unsigned char *bits;
     int rowstride;
 
-    surface = malloc (sizeof (*surface));
+    surface = _cairo_malloc (sizeof (*surface));
     if (surface == NULL)
 	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_NO_MEMORY));
 
@@ -325,6 +322,8 @@ _cairo_win32_display_surface_create_for_dc (HDC             original_dc,
     surface->win32.extents.y = 0;
     surface->win32.extents.width = width;
     surface->win32.extents.height = height;
+    surface->win32.x_ofs = 0;
+    surface->win32.y_ofs = 0;
 
     surface->initial_clip_rgn = NULL;
     surface->had_simple_clip = FALSE;
@@ -466,7 +465,8 @@ _cairo_win32_display_surface_map_to_image (void                    *abstract_sur
 		     surface->win32.extents.width,
 		     surface->win32.extents.height,
 		     surface->win32.dc,
-		     surface->win32.extents.x, surface->win32.extents.y,
+		     surface->win32.extents.x + surface->win32.x_ofs, /* Handling multi-monitor... */
+		     surface->win32.extents.y + surface->win32.y_ofs, /* ... setup on Win32 */
 		     SRCCOPY)) {
 	    status = _cairo_error (CAIRO_STATUS_DEVICE_ERROR);
 	    goto err;
@@ -544,11 +544,12 @@ _cairo_win32_display_surface_flush (void *abstract_surface, unsigned flags)
 
 	if (damage->status) {
 	    if (!BitBlt (surface->win32.dc,
-			 0, 0,
+			 surface->win32.extents.x + surface->win32.x_ofs, /* Handling multi-monitor... */
+			 surface->win32.extents.y + surface->win32.y_ofs, /* ... setup on Win32 */
 			 surface->win32.extents.width,
 			 surface->win32.extents.height,
 			 fallback->win32.dc,
-			 0, 0,
+			 surface->win32.extents.x, surface->win32.extents.y,
 			 SRCCOPY))
 		status = _cairo_win32_print_gdi_error (__FUNCTION__);
 	} else if (damage->region) {
@@ -561,7 +562,8 @@ _cairo_win32_display_surface_flush (void *abstract_surface, unsigned flags)
 			rect.x, rect.y,
 			rect.width, rect.height));
 		if (!BitBlt (surface->win32.dc,
-			     rect.x, rect.y,
+			     rect.x + surface->win32.x_ofs, /* Handling multi-monitor... */
+			     rect.y + surface->win32.y_ofs, /* ... setup on Win32 */
 			     rect.width, rect.height,
 			     fallback->win32.dc,
 			     rect.x, rect.y,
@@ -628,6 +630,35 @@ _cairo_win32_save_initial_clip (HDC hdc, cairo_win32_display_surface_t *surface)
     surface->win32.extents.width = rect.right - rect.left;
     surface->win32.extents.height = rect.bottom - rect.top;
 
+    /* On multi-monitor setup, under Windows, the primary monitor always
+     * have origin (0,0).  Any monitors that extends to the left or above
+     * will have coordinates in the negative range.  Take this into
+     * account, by forcing our Win32 surface to start at extent (0,0) and
+     * using a device offset.  Cairo does not handle extents with negative
+     * offsets.
+     */
+    surface->win32.x_ofs = 0;
+    surface->win32.y_ofs = 0;
+    if ((surface->win32.extents.x < 0) ||
+	(surface->win32.extents.y < 0)) {
+	/* Negative offsets occurs for (and ONLY for) the desktop DC (virtual
+	 * desktop), when a monitor extend to the left or above the primary
+	 * monitor.
+	 *
+	 * More info @ https://www.microsoft.com/msj/0697/monitor/monitor.aspx
+	 *
+	 * Note that any other DC, including memory DC created with
+	 * CreateCompatibleDC(<virtual desktop DC>) will have extents in the
+	 * positive range.  This will be taken into account later when we perform
+	 * raster operations between the DC (may have to perform offset
+	 * translation).
+	 */
+	surface->win32.x_ofs = surface->win32.extents.x;
+	surface->win32.y_ofs = surface->win32.extents.y;
+	surface->win32.extents.x = 0;
+	surface->win32.extents.y = 0;
+    }
+
     surface->initial_clip_rgn = NULL;
     surface->had_simple_clip = FALSE;
 
@@ -685,7 +716,7 @@ _cairo_win32_display_surface_set_clip (cairo_win32_display_surface_t *surface,
 
     data_size = sizeof (RGNDATAHEADER) + num_rects * sizeof (RECT);
     if (data_size > sizeof (stack)) {
-	data = malloc (data_size);
+	data = _cairo_malloc (data_size);
 	if (!data)
 	    return _cairo_error(CAIRO_STATUS_NO_MEMORY);
     } else
@@ -947,14 +978,21 @@ cairo_win32_surface_create_with_format (HDC hdc, cairo_format_t format)
     cairo_device_t *device;
 
     switch (format) {
-    default:
-	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_INVALID_FORMAT));
-    case CAIRO_FORMAT_ARGB32:
-    case CAIRO_FORMAT_RGB24:
-	break;
+	case CAIRO_FORMAT_INVALID:
+	case CAIRO_FORMAT_A8:
+	case CAIRO_FORMAT_A1:
+	case CAIRO_FORMAT_RGB16_565:
+	case CAIRO_FORMAT_RGB30:
+	case CAIRO_FORMAT_RGB96F:
+	case CAIRO_FORMAT_RGBA128F:
+	default:
+	    return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_INVALID_FORMAT));
+	case CAIRO_FORMAT_ARGB32:
+	case CAIRO_FORMAT_RGB24:
+	    break;
     }
 
-    surface = malloc (sizeof (*surface));
+    surface = _cairo_malloc (sizeof (*surface));
     if (surface == NULL)
 	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_NO_MEMORY));
 
@@ -973,7 +1011,7 @@ cairo_win32_surface_create_with_format (HDC hdc, cairo_format_t format)
     surface->is_dib = FALSE;
     surface->saved_dc_bitmap = NULL;
 
-    surface->win32.flags = _cairo_win32_flags_for_dc (surface->win32.dc);
+    surface->win32.flags = _cairo_win32_flags_for_dc (surface->win32.dc, format);
 
     device = _cairo_win32_device_get ();
 
@@ -1062,14 +1100,19 @@ cairo_win32_surface_create_with_ddb (HDC hdc,
     HBITMAP saved_dc_bitmap;
 
     switch (format) {
-    default:
 /* XXX handle these eventually */
-    case CAIRO_FORMAT_A8:
-    case CAIRO_FORMAT_A1:
-	return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_INVALID_FORMAT));
-    case CAIRO_FORMAT_ARGB32:
-    case CAIRO_FORMAT_RGB24:
-	break;
+	case CAIRO_FORMAT_INVALID:
+	case CAIRO_FORMAT_A8:
+	case CAIRO_FORMAT_A1:
+	case CAIRO_FORMAT_RGB16_565:
+	case CAIRO_FORMAT_RGB30:
+	case CAIRO_FORMAT_RGB96F:
+	case CAIRO_FORMAT_RGBA128F:
+	default:
+	    return _cairo_surface_create_in_error (_cairo_error (CAIRO_STATUS_INVALID_FORMAT));
+	case CAIRO_FORMAT_ARGB32:
+	case CAIRO_FORMAT_RGB24:
+	    break;
     }
 
     if (!hdc) {
